package jalview.ws.jws2;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.ConnectException;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import org.apache.log4j.Level;

import jalview.bin.Cache;
import jalview.datamodel.AlignmentView;
import jalview.gui.AlignFrame;
import jalview.ws.WSMenuEntryProviderI;
import compbio.data.msa.MsaWS;
import compbio.metadata.Preset;
import compbio.metadata.PresetManager;
import compbio.ws.client.Jws2Base;
import compbio.ws.client.Jws2Base.Services;

/**
 * discoverer for jws2 services. Follows the lightweight service discoverer
 * pattern (archetyped by EnfinEnvision2OneWay)
 * 
 * @author JimP
 * 
 */
public class Jws2Discoverer implements Runnable, WSMenuEntryProviderI
{
  private java.beans.PropertyChangeSupport changeSupport = new java.beans.PropertyChangeSupport(
          this);

  /**
   * change listeners are notified of "services" property changes
   * 
   * @param listener
   *          to be added that consumes new services Hashtable object.
   */
  public void addPropertyChangeListener(
          java.beans.PropertyChangeListener listener)
  {
    changeSupport.addPropertyChangeListener(listener);
  }

  /**
   * 
   * 
   * @param listener
   *          to be removed
   */
  public void removePropertyChangeListener(
          java.beans.PropertyChangeListener listener)
  {
    changeSupport.removePropertyChangeListener(listener);
  }

  boolean running = false;

  @Override
  public void run()
  {
    if (running)
    {
      return;
    }
    running = true;
    try
    {
      Class foo = getClass().getClassLoader().loadClass(
              "compbio.ws.client.JwsBase");
    } catch (ClassNotFoundException e)
    {
      System.err
              .println("Not enabling Jalview Webservices version 2: client jar is not available."
                      +"\nPlease check that your webstart JNLP file is up to date!");
      running = false;
      return;
    }
    // Cache.initLogger();
    // Cache.log.setLevel(Level.DEBUG);
    // TODO: Document and PACK JWS2
    String jwsservers = Cache.getDefault("JWS2HOSTURLS",
            "http://webservices.compbio.dundee.ac.uk:8084/jws2");
    try
    {
      if (Jws2Base.validURL(jwsservers))
      {
        // look for services
        for (Services srv : Jws2Base.Services.values())
        {
          MsaWS service = null;
          try
          {
            service = Jws2Base.connect(jwsservers, srv);
          } catch (Exception e)
          {
            System.err.println("Jws2 Discoverer: Problem with "
                    + jwsservers + " with service " + srv + ":\n"
                    + e.getMessage());
            if (!(e instanceof javax.xml.ws.WebServiceException))
            {
              e.printStackTrace();
            }
          }
          ;
          if (service != null)
          {
            addService(jwsservers, srv, service);
          }
        }

      }
      else
      {
        Cache.log.info("Ignoring invalid Jws2 service url " + jwsservers);
      }
    } catch (Exception e)
    {
      e.printStackTrace();
      Cache.log.warn("Exception when discovering Jws2 services.", e);
    } catch (Error e)
    {
      Cache.log.error("Exception when discovering Jws2 services.", e);
    }
    running = false;
    changeSupport.firePropertyChange("services", new Vector(), services);
  }

  /**
   * record this service endpoint so we can use it
   * 
   * @param jwsservers
   * @param srv
   * @param service2
   */
  private void addService(String jwsservers, Services srv, MsaWS service2)
  {
    if (services == null)
    {
      services = new Vector<Jws2Instance>();
    }
    System.out.println("Discovered service: " + jwsservers + " "
            + srv.toString());
    services.add(new Jws2Instance(jwsservers, "Align with "
            + srv.toString(), service2));
  }

  public class Jws2Instance
  {
    String hosturl;

    String serviceType;

    MsaWS service;

    public Jws2Instance(String hosturl, String serviceType, MsaWS service)
    {
      super();
      this.hosturl = hosturl;
      this.serviceType = serviceType;
      this.service = service;
    }

    PresetManager presets = null;

    /**
     * non thread safe - gets the presets for this service (blocks whilst it
     * calls the service to get the preset set)
     * 
     * @return service presets or null if exceptions were raised.
     */
    public PresetManager getPresets()
    {
      if (presets == null)
      {
        try
        {
          presets = service.getPresets();
        } catch (Exception ex)
        {
          System.err
                  .println("Exception when retrieving presets for service "
                          + serviceType + " at " + hosturl);
        }
      }
      return presets;
    }
  };

  /**
   * holds list of services.
   */
  Vector<Jws2Instance> services;

  @Override
  public void attachWSMenuEntry(JMenu wsmenu, final AlignFrame alignFrame)
  {
    if (running || services == null || services.size() == 0)
    {
      return;
    }
    /**
     * eventually, JWS2 services will appear under the same align/etc submenus.
     * for moment we keep them separate.
     */
    JMenu jws2 = new JMenu("JWS2 Alignment");
    MsaWSClient msacl = new MsaWSClient();
    for (final Jws2Instance service : services)
    {
      msacl.attachWSMenuEntry(jws2, service, alignFrame);
      /*
       * JMenuItem sitem = new JMenuItem(service.serviceType);
       * sitem.setToolTipText("Hosted at " + service.hosturl);
       * sitem.addActionListener(new ActionListener() {
       * 
       * @Override public void actionPerformed(ActionEvent e) { AlignmentView
       * msa = alignFrame.gatherSequencesForAlignment(); MsaWSClient client =
       * new MsaWSClient(service, "JWS2 Alignment of " + alignFrame.getTitle(),
       * msa, false, true, alignFrame.getViewport().getAlignment().getDataset(),
       * alignFrame); } });
       */
    }
    if (services.size() > 0)
    {
      wsmenu.add(jws2);
    }

  }

  public static void main(String[] args)
  {
    Thread runner = new Thread(getDiscoverer());
    getDiscoverer().addPropertyChangeListener(new PropertyChangeListener()
    {

      @Override
      public void propertyChange(PropertyChangeEvent evt)
      {
        System.out.println("Changesupport: There are now "
                + getDiscoverer().services.size() + " services");
      }
    });
    runner.start();
    while (runner.isAlive())
    {
      try
      {
        Thread.sleep(50);
      } catch (InterruptedException e)
      {
      }
      ;
    }
  }

  private static Jws2Discoverer discoverer;

  public static Jws2Discoverer getDiscoverer()
  {
    if (discoverer == null)
    {
      discoverer = new Jws2Discoverer();
    }
    return discoverer;
  }

  public boolean hasServices()
  {
    // TODO Auto-generated method stub
    return !running && services != null && services.size() > 0;
  }

  public boolean isRunning()
  {
    return running;
  }

}
